-- Test surroundings_highlight
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(50);

set search_path to camper, public;

select has_table('surroundings_highlight');
select has_pk('surroundings_highlight');
select table_privs_are('surroundings_highlight', 'guest', array['SELECT']);
select table_privs_are('surroundings_highlight', 'employee', array['SELECT']);
select table_privs_are('surroundings_highlight', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('surroundings_highlight', 'authenticator', array[]::text[]);

select has_column('surroundings_highlight', 'surroundings_highlight_id');
select col_is_pk('surroundings_highlight', 'surroundings_highlight_id');
select col_type_is('surroundings_highlight', 'surroundings_highlight_id', 'integer');
select col_not_null('surroundings_highlight', 'surroundings_highlight_id');
select col_hasnt_default('surroundings_highlight', 'surroundings_highlight_id');

select has_column('surroundings_highlight', 'company_id');
select col_is_fk('surroundings_highlight', 'company_id');
select fk_ok('surroundings_highlight', 'company_id', 'company', 'company_id');
select col_type_is('surroundings_highlight', 'company_id', 'integer');
select col_not_null('surroundings_highlight', 'company_id');
select col_hasnt_default('surroundings_highlight', 'company_id');

select has_column('surroundings_highlight', 'media_id');
select col_type_is('surroundings_highlight', 'media_id', 'integer');
select col_not_null('surroundings_highlight', 'media_id');
select col_hasnt_default('surroundings_highlight', 'media_id');

select has_column('surroundings_highlight', 'name');
select col_is_fk('surroundings_highlight', 'media_id');
select fk_ok('surroundings_highlight', 'media_id', 'media', 'media_id');
select col_type_is('surroundings_highlight', 'name', 'text');
select col_not_null('surroundings_highlight', 'name');
select col_hasnt_default('surroundings_highlight', 'name');

select has_column('surroundings_highlight', 'description');
select col_type_is('surroundings_highlight', 'description', 'xml');
select col_not_null('surroundings_highlight', 'description');
select col_hasnt_default('surroundings_highlight', 'description');

select has_column('surroundings_highlight', 'position');
select col_type_is('surroundings_highlight', 'position', 'integer');
select col_not_null('surroundings_highlight', 'position');
select col_has_default('surroundings_highlight', 'position');
select col_default_is('surroundings_highlight', 'position', '2147483647');


set client_min_messages to warning;
truncate surroundings_highlight cascade;
truncate media cascade;
truncate media_content cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into media_content (media_type, bytes)
values ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
;

insert into media (media_id, company_id, original_filename, content_hash)
values (6, 2, 'cover2.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
     , (8, 4, 'cover4.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
;

insert into surroundings_highlight (company_id, media_id, name, description)
values (2, 6, 'Girona', '<p>Beautiful</p>')
     , (4, 8, 'Barcelona', '<p>Shit</p>')
;

prepare surroundings_highlight_data as
select company_id, name
from surroundings_highlight
order by company_id, name;

set role guest;
select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (4, 'Barcelona')
	$$,
	'Everyone should be able to list all surroundings highlights across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into surroundings_highlight(company_id, media_id, name, description) values (2, 6, 'Another highlight', '<div/>') $$,
	'Admin from company 2 should be able to insert a new surroundings highlights to that company.'
);

select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (2, 'Another highlight')
	        , (4, 'Barcelona')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update surroundings_highlight set name = 'Another' where company_id = 2 and name = 'Another highlight' $$,
	'Admin from company 2 should be able to update surroundings highlights of that company.'
);

select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (2, 'Another')
	        , (4, 'Barcelona')
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from surroundings_highlight where company_id = 2 and name = 'Another' $$,
	'Admin from company 2 should be able to delete surroundings highlights from that company.'
);

select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (4, 'Barcelona')
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into surroundings_highlight (company_id, media_id, name, description) values (4, 8, 'Another highlight', '<div/>') $$,
	'42501', 'new row violates row-level security policy for table "surroundings_highlight"',
	'Admin from company 2 should NOT be able to insert new surroundings highlights to company 4.'
);

select lives_ok(
	$$ update surroundings_highlight set name = 'Nope' where company_id = 4 $$,
	'Admin from company 2 should not be able to update new surroundings highlights of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (4, 'Barcelona')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update surroundings_highlight set company_id = 4 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "surroundings_highlight"',
	'Admin from company 2 should NOT be able to move surroundings highlights to company 4'
);

select lives_ok(
	$$ delete from surroundings_highlight where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete surroundings highlights from company 4, but not error is thrown'
);

select bag_eq(
	'surroundings_highlight_data',
	$$ values (2, 'Girona')
	        , (4, 'Barcelona')
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into surroundings_highlight (company_id, media_id, name, description) values (2, 6, '  ', '') $$,
	'23514', 'new row for relation "surroundings_highlight" violates check constraint "name_not_empty"',
	'Should not be able to insert surroundings highlights with a blank name.'
);


reset role;


select *
from finish();

rollback;

